#include "runtime/gfx.h"
#include "runtime/resources.h"
#include "runtime/mem_arena.h"

#include "asset_compiler/asset_compiler.h"

static rt_framegraph *_framegraph;

void RegisterCVars(void) {
    rtRegisterAssetCompilerCVars();
}

/* Called after the runtime has finished its initialization and before entering the main-loop*/
void Init(void) {
    rtLog("GAME", "Init");

    rtInitAssetCompiler();

    #if 0
    rt_render_target_id rt_ids[4] = {rtCalculateRenderTargetID("rt0", sizeof("rt0")),
                                     rtCalculateRenderTargetID("rt1", sizeof("rt1")),
                                     rtCalculateRenderTargetID("rt2", sizeof("rt2")),
                                     rtCalculateRenderTargetID("rt3", sizeof("rt3")),
    };
    
    rt_render_target_info rts[4] = {
        {.id = rt_ids[0], .width = 1024, .height = 768, .format = RT_PIXEL_FORMAT_R8G8B8A8_SRGB, .sample_count = 4},
        {.id = rt_ids[1], .width = 1024, .height = 768, .format = RT_PIXEL_FORMAT_R8G8B8A8_SRGB, .sample_count = 4},
        {.id = rt_ids[2], .width = 1024, .height = 768, .format = RT_PIXEL_FORMAT_R8G8B8A8_SRGB, .sample_count = 4},
        {.id = rt_ids[3], .width = 1024, .height = 768, .format = RT_PIXEL_FORMAT_R8G8B8A8_SRGB, .sample_count = 4}
    };

    rt_render_target_write pass0_writes[] = {{.render_target = rt_ids[0]},
                                             {.render_target = rt_ids[1]}};
    rt_render_target_read pass1_reads[]   = {{.render_target = rt_ids[0]},
                                             {.render_target = rt_ids[1]}};
    rt_render_target_write pass1_writes[] = {{.render_target = rt_ids[2]}};
    rt_render_target_read pass2_reads[]   = {{.render_target = rt_ids[2]},
                                             {.render_target = rt_ids[1]}};
    rt_render_target_write pass2_writes[] = {{.render_target = rt_ids[3]}};
    rt_render_target_write pass3_writes[] = {{.render_target = rt_ids[2]}};

    rt_render_pass_info passes[4];
    rtSetRelptr(&passes[0].read_render_targets, NULL);
    rtSetRelptr(&passes[0].write_render_targets, pass0_writes);
    passes[0].read_render_target_count = 0;
    passes[0].write_render_target_count = RT_ARRAY_COUNT(pass0_writes);
    rtSetRelptr(&passes[1].read_render_targets, pass1_reads);
    rtSetRelptr(&passes[1].write_render_targets, pass1_writes);
    passes[1].read_render_target_count = RT_ARRAY_COUNT(pass1_reads);
    passes[1].write_render_target_count = RT_ARRAY_COUNT(pass1_writes);
    rtSetRelptr(&passes[2].read_render_targets, pass2_reads);
    rtSetRelptr(&passes[2].write_render_targets, pass2_writes);
    passes[2].read_render_target_count  = RT_ARRAY_COUNT(pass2_reads);
    passes[2].write_render_target_count = RT_ARRAY_COUNT(pass2_writes);
    rtSetRelptr(&passes[3].read_render_targets, NULL);
    rtSetRelptr(&passes[3].write_render_targets, pass3_writes);
    passes[3].read_render_target_count  = 0;
    passes[3].write_render_target_count = RT_ARRAY_COUNT(pass3_writes);


    rt_framegraph_info info = {
        .render_pass_count = 4,
        .render_target_count = 4,
    };
    rtSetRelptr(&info.render_passes, passes);
    rtSetRelptr(&info.render_targets, rts);
    #endif

    rt_resource_id id  = rtGetResourceID("assets/test.framegraph");
    rt_temp_arena temp = rtGetTemporaryArena(NULL, 0);
    while (rtGetResourceSize(id) == 0)
        ;
    rt_resource *resource =
        rtArenaPush(temp.arena, rtGetResourceSize(id));
    
    rtGetResource(id, resource);

    _framegraph = rtCreateFramegraph(resource->data);
}

/* Called after exiting the main-loop and before the runtime starts its shutdown */
void Shutdown(void) {
    rtLog("GAME", "Shutdown");
    rtShutdownAssetCompiler();
    rtDestroyFramegraph(_framegraph);
}